<?php
namespace ExtDev\Selfregistration\Controller;

/***************************************************************
 *
 * This is free and unencumbered software
 * released into the public domain.
 *
 * (C0) 2018 ext/dev <info@extdev.io>
 *
 ***************************************************************/

use TYPO3\CMS\Core\Page\PageRenderer;
use TYPO3\CMS\Core\Utility\GeneralUtility;

class FrontendUserController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{

    /**
    * frontendUserGroupRepository
    *
    * @var \TYPO3\CMS\Extbase\Domain\Repository\FrontendUserGroupRepository
    * @inject
    */
    protected $frontendUserGroupRepository = NULL;

    /**
    * frontendUserRepository
    *
    * @var \ExtDev\Selfregistration\Domain\Repository\FrontendUserRepository
    * @inject
    */
    protected $frontendUserRepository = NULL;

    /**
    * action ajaxIsUsernameValid
    *
    * @param string $username
    * @return void
    */
    public function ajaxIsUsernameValidAction(String $username)
    {
        $response = array(
            'usernameValid' => true,
            'help' => \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate( 'tx_selfregistration.help.valid_username', 'selfregistration')
        );

        // check if username exists
        $frontendUsers = $this->frontendUserRepository->findByUsername($username);
        if(count($frontendUsers->toArray()) > 0 ) {
            $response = array(
                'usernameValid' => false,
                'reason' => 'userExists',
                'help' => \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate( 'tx_selfregistration.help.user_exists', 'selfregistration')
            );
        }
        // check if email = username
        else if($this->settings['useEmailAsUsername']) {
            // check if valid email address
            if (!filter_var($username, FILTER_VALIDATE_EMAIL)) {
                $response = array(
                    'usernameValid' => false,
                    'reason' => 'invalidEmail',
                    'help' => \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate( 'tx_selfregistration.help.invalid_email', 'selfregistration')
                );
            }
        }

        $this->response->setHeader('Content-Type', 'text/json', true);
        $this->response->sendHeaders();
        echo json_encode($response);
        die;
    }

    /**
    * action confirmEmail
    *
    * @param string $verificationCode
    * @return void
    */
    public function confirmEmailAction(String $verificationCode)
    {
        $frontendUser = $this->frontendUserRepository->findByVerificationCode($verificationCode);
        if($frontendUser) {
            $frontendUser->setDisable(false);
            $frontendUser->setVerificationCode('');
            $this->frontendUserRepository->update($frontendUser);

            $persistenceManager = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Persistence\\Generic\\PersistenceManager');
            $persistenceManager->persistAll();

            if($this->request->hasArgument('autoLogin') && $this->request->getArgument('autoLogin')) {
                $GLOBALS['TSFE']->fe_user->checkPid = false;
                $GLOBALS['TSFE']->fe_user->checkPid_value = '';

                // the user is supposed to get logged in...
                $authInfo = $GLOBALS['TSFE']->fe_user->getAuthInfoArray();
                $user_db = $GLOBALS['TSFE']->fe_user->fetchUserRecord($authInfo['db_user'], $frontendUser->getUsername());
                if($user_db) {
                    $GLOBALS['TSFE']->fe_user->forceSetCookie = TRUE;
                    $GLOBALS['TSFE']->fe_user->createUserSession($user_db);
                    $GLOBALS['TSFE']->initUserGroups();
                    $GLOBALS['TSFE']->fe_user->user = $user_db;
                    $GLOBALS['TSFE']->fe_user->setKey('ses', 'fe_typo_user', $user_db);
                    $GLOBALS['TSFE']->loginUser = 1;
                    $GLOBALS['TSFE']->fe_user->setAndSaveSessionData('dummy', TRUE);
                    setcookie('fe_typo_user', $GLOBALS['TSFE']->fe_user->user['ses_id'], NULL, '/');
                }
            }

            if($this->request->hasArgument('redirect') && is_array($this->request->getArgument('redirect'))) {
                $redirect = $this->request->getArgument('redirect');
                $uri = $this->uriBuilder->reset()
                  ->setTargetPageUid($redirect['pageUid'] ? $redirect['pageUid'] : $GLOBALS['TSFE']->page['uid'])
                  ->setArguments($redirect['arguments'])
                  ->setCreateAbsoluteUri(true)
                  ->build();
                $this->redirectToURI($uri);
            } else {
                $redirectPageUid = $this->settings['validationSuccessfulPid'] ? $this->settings['validationSuccessfulPid'] : NULL;
                // redirect to validationSuccessfulAction
                $this->redirect(
                    'validationSuccessful',
                    NULL,
                    NULL,
                    NULL,
                    $redirectPageUid
                );
            }
        } else {
            // redirect to validationFailedAction
            $this->redirect(
                'validationFailed',
                NULL,
                NULL,
                NULL,
                ($this->settings['validationFailedPid'] ? $this->settings['validationFailedPid'] : NULL)
            );
        }

    }

    /**
    * action create
    *
    * @param \ExtDev\Selfregistration\Domain\Model\FrontendUser $frontendUser
    * @return void
    */
    public function createAction(\ExtDev\Selfregistration\Domain\Model\FrontendUser $frontendUser)
    {
        if($this->settings['useEmailAsUsername']) {
            $frontendUser->setEmail($frontendUser->getUsername());
        }

        // check for additionalData
        $additionalData = array();
        if($this->request->hasArgument('additionalData')) {
            $additionalData = $this->request->getArgument('additionalData');
        }

        // instantiate CreateHelper Class
        $createHelper = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('ExtDev\\Selfregistration\\Helper\\FrontendUser\\CreateHelper');

        // try to create frontendUser with CreateHelper
        $createResponse = $createHelper->createFrontendUser($frontendUser, $this->settings, $additionalData);
        if($createResponse['validUserData']) {
            if($this->settings['requireEmailValidation']) {
                // redirect to validationRequiredAction
                $this->redirect(
                    'validationRequired',
                    NULL,
                    NULL,
                    NULL,
                    ($this->settings['validationRequiredPid'] ? $this->settings['validationRequiredPid'] : NULL)
                );
            } else {
                // redirect to createSuccessfulAction
                $this->redirect(
                    'createSuccessful',
                    NULL,
                    NULL,
                    NULL,
                    ($this->settings['createSuccessfulPid'] ? $this->settings['createSuccessfulPid'] : NULL)
                );
            }
        } else {
            // redirect to self to show form again
            $this->redirect(
                'new',
                NULL,
                NULL,
                array(
                    'errors' => $createResponse['errors'],
                    'values' => array(
                        'firstName' => $frontendUser->getFirstName(),
                        'lastName' => $frontendUser->getLastName() ,
                        'username' => $frontendUser->getUsername()
                    )
                ),
                NULL
            );
        }
    }

    /**
    * action createFailed
    *
    * @return void
    */
    public function createFailedAction()
    {
    }

    /**
    * action createSuccessful
    *
    * @return void
    */
    public function createSuccessfulAction()
    {
    }

    /**
    * action new
    *
    * @return void
    */
    public function newAction()
    {
        // check for additionalData
        $errors = array();
        if($this->request->hasArgument('errors')) {
            $errors = $this->request->getArgument('errors');
            $this->view->assign('errors', $errors);
        }
        // check for values
        if($this->request->hasArgument('values')) {
            $values = $this->request->getArgument('values');
            $this->view->assign('values', $values);
        }

        $pageRenderer = GeneralUtility::makeInstance(PageRenderer::class);
        $pageRenderer->addJsFile(
            'typo3conf/ext/selfregistration/Resources/Public/JavaScript/LoadRsaEncryption.js',
            'text/javascript'
        );
        $pageRenderer->addJsFooterFile(
            'typo3conf/ext/selfregistration/Resources/Public/JavaScript/ValidateForm.js',
            'text/javascript'
        );
        $this->view->assign('termsConditionsPid', $this->settings['termsConditionsPid']);
    }

    /**
    * action validationFailed
    *
    * @return void
    */
    public function validationFailedAction()
    {
    }

    /**
    * action validationRequired
    *
    * @return void
    */
    public function validationRequiredAction()
    {
    }

    /**
    * action validationSuccesful
    *
    * @return void
    */
    public function validationSuccessfulAction()
    {
    }

}
