<?php
namespace ExtDev\Selfregistration\Helper\FrontendUser;

/***************************************************************
 *
 * This is free and unencumbered software
 * released into the public domain.
 *
 * (C0) 2018 ext/dev <info@extdev.io>
 *
 ***************************************************************/

use TYPO3\CMS\Rsaauth\Backend\BackendFactory;
use TYPO3\CMS\Rsaauth\Hook\FrontendLoginHook;
use TYPO3\CMS\Rsaauth\Storage\StorageFactory;

class CreateHelper {

    /**
    * frontendUserGroupRepository
    *
    * @var \TYPO3\CMS\Extbase\Domain\Repository\FrontendUserGroupRepository
    * @inject
    */
    protected $frontendUserGroupRepository = NULL;

    /**
    * frontendUserRepository
    *
    * @var \ExtDev\Selfregistration\Domain\Repository\FrontendUserRepository
    * @inject
    */
    protected $frontendUserRepository = NULL;

    /**
    * frontendUser
    *
    * @var \ExtDev\Selfregistration\Domain\Model\FrontendUser
    */
    public $frontendUser = NULL;

    /**
    * frontendUser
    *
    * @var \Array
    */
    public $additionalData = NULL;

    protected $settings = array();

    public function createFrontendUser(\ExtDev\Selfregistration\Domain\Model\FrontendUser $frontendUser, $settings = array(), $additionalData = array()) {
        $this->frontendUser = $frontendUser;

        if(is_array($settings)) {
            $this->settings = $settings;
        }

        if(is_array($additionalData)) {
            $this->additionalData = $additionalData;
        }

        $objectManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\\CMS\\Extbase\\Object\\ObjectManager');

        $this->frontendUserGroupRepository = $objectManager->get('TYPO3\\CMS\\Extbase\\Domain\\Repository\\FrontendUserGroupRepository');

        $this->frontendUserRepository = $objectManager->get('TYPO3\\CMS\\Extbase\\Domain\\Repository\\FrontendUserRepository');

        // initialize response
        $response = array();

        // assume valid user data
        $response['validUserData'] = true;

        $passwordsToDecrypt['originalPassword'] = $this->frontendUser->getPassword();


        // if repeated password was given add it do passwordsToDecrypt
        if($this->additionalData['repeatPassword']) {
            $passwordsToDecrypt['repeatedPassword'] = $this->additionalData['repeatPassword'];
        }

        // decrypt Passwords
        $decryptedPasswords = $this->decrypt($passwordsToDecrypt);

        // assign decryptedPassword
        $decryptedPassword = $decryptedPasswords['originalPassword'];

        if(
            $this->additionalData['repeatPassword'] &&
            $decryptedPasswords['originalPassword'] !=  $decryptedPasswords['repeatedPassword']
        ) {
                $response['validUserData'] = false;
                $response['errors']['additionalData']['repeatPassword'] = true;
        }

        // create salted password
        $saltedPassword = '';
        $objSalt = \TYPO3\CMS\Saltedpasswords\Salt\SaltFactory::getSaltingInstance(NULL);
        if (is_object($objSalt)) {
            $saltedPassword = $objSalt->getHashedPassword($decryptedPassword);
            $this->frontendUser->setPassword($saltedPassword);
        } else {
            $response['validUserData'] = false;
            $response['errors']['password']['creationFailed'] = true;
        }

        // check if user exists
        if(count($this->frontendUserRepository->findByUsername($this->frontendUser->getUsername())->toArray()) > 0) {
            $response['validUserData'] = false;
            $response['errors']['username']['usernameExists'] = true;
        }

        // check if email is valid
        if(!filter_var($this->frontendUser->getEmail(), FILTER_VALIDATE_EMAIL)) {
            $response['validUserData'] = false;
            $response['errors']['username']['invalidEmailAddress'] = true;
        }

        // check each mandatory additional field
        if(count($this->additionalData['mandatory'])) {
            foreach ($this->additionalData['mandatory'] as $key => $value) {
                if(!$value) {
                    $response['validUserData'] = false;
                    $response['errors']['additionalData'][$key] = true;
                }
            }
        }

        // see if user data is valid
        if($response['validUserData']) {

            // add default usergroup
            if($this->settings['defaultUserGroup']) {
                $frontendUserGroup = $this->frontendUserGroupRepository->findByUid($this->settings['defaultUserGroup']);
                $this->frontendUser->addUserGroup($frontendUserGroup);
            }

            // check if email validation is required
            if($this->settings['requireEmailValidation']) {
                // email validation is required

                // we set the user as disabled
                $this->frontendUser->setDisable(true);

                // we create a secure random string as auth code
                $verificationCode = $this->getRandomString();
                $this->frontendUser->setVerificationCode($verificationCode);

                // create email from locallang
                $mailSubject = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('tx_selfregistration.validationMail.subject','selfregistration');
                $mailBody = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\CMS\Fluid\View\StandaloneView');
                $mailBody->setFormat('html');
                $mailBody->setTemplatePathAndFilename($settings['validationMailTemplateFile']);
                $mailBody->assign('frontendUser',$this->frontendUser);

                if($this->additionalData['autoLogin']) {
                    $mailBody->assign('autoLogin',$this->additionalData['autoLogin']);
                }

                if($this->additionalData['redirect']){
                    $mailBody->assign('redirect',$this->additionalData['redirect']);
                }

                if($this->additionalData['confirmationPageUid']){
                    $mailBody->assign('confirmationPageUid',$this->additionalData['confirmationPageUid']);
                }

                // send email
                $mail = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\\CMS\\Core\\Mail\\MailMessage');
                $mail
                    ->setSubject($mailSubject)
                    ->setTo(array($this->frontendUser->getEmail() => $this->frontendUser->getFirstName() . ' ' .$this->frontendUser->getLastName()))
                    ->setBody($mailBody->render(),'text/html')
                    ->send();
            }

            // user was created - execute createHook
            if (is_array($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['ext/selfregistration/Classes/Helper/FrontendUser/CreateHelper']['createFrontendUserPostProc'])) {
                $_params = array();
                foreach($GLOBALS['TYPO3_CONF_VARS']['SC_OPTIONS']['ext/selfregistration/Classes/Helper/FrontendUser/CreateHelper']['createFrontendUserPostProc'] as $_funcRef) {
                    \TYPO3\CMS\Core\Utility\GeneralUtility::callUserFunction($_funcRef, $_params, $this);
                }
            }

            // persist frontendUser
            $this->frontendUserRepository->add($this->frontendUser);
            return $response;
        } else {
            return $response;
        }
    }

    private function getRandomString($length = 64)
    {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    private function decrypt(array $data) {
        switch (self::getTransmissionSecurityLevel()) {
            case 'rsa':
                // Get services from rsaauth
                $backend = BackendFactory::getBackend();
                $storage = StorageFactory::getStorage();
                if (is_object($backend) && is_object($storage)) {
                    $key = $storage->get();
                    if ($key !== null) {
                        if (isset($data) && $data !== '') {
                            foreach($data as $k=>$v) {
                                if (substr($v, 0, 4) === 'rsa:') {
                                    // Decode passwords
                                    $result[$k] = $backend->decrypt($key, substr($v, 4));
                                }
                            }
                            if ($result !== null) {
                                // Remove the key
                                $storage->put(null);
                                //return result
                                return $result;
                            }
                        }
                    } else {
                        // RSA auth service failed to retrieve private key
                        // May happen if the key was already removed
                        $success = false;
                        $message = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('internal_rsaauth_retrieve_private_key_failed', 'selfregistration');
                        \TYPO3\CMS\Core\Utility\GeneralUtility::sysLog($message, 'selfregistration', \TYPO3\CMS\Core\Utility\GeneralUtility::SYSLOG_SEVERITY_WARNING);
                    }
                } else {
                    // Required RSA auth backend not available
                    // Should not happen
                    $success = false;
                    $message = \TYPO3\CMS\Extbase\Utility\LocalizationUtility::translate('internal_rsaauth_backend_not_available', 'selfregistration');
                    \TYPO3\CMS\Core\Utility\GeneralUtility::sysLog($message, 'selfregistration', \TYPO3\CMS\Core\Utility\GeneralUtility::SYSLOG_SEVERITY_ERROR);
                }
                break;
            case 'normal':
            default:
                // Nothing to decrypt
                return $data;
                break;
        }
    }

    /**
     * Gets the transmission security level
     *
     * @return string the transmission security level
     */
    static public function getTransmissionSecurityLevel()
    {
        return $GLOBALS['TYPO3_CONF_VARS']['FE']['loginSecurityLevel'];
    }
}
